@@ -1,27 +1,45 @@
 # -*- coding: utf-8 -*-
+#  Licensed to Elasticsearch B.V. under one or more contributor
+#  license agreements. See the NOTICE file distributed with
+#  this work for additional information regarding copyright
+#  ownership. Elasticsearch B.V. licenses this file to you under
+#  the Apache License, Version 2.0 (the "License"); you may
+#  not use this file except in compliance with the License.
+#  You may obtain a copy of the License at
+#
+# 	http://www.apache.org/licenses/LICENSE-2.0
+#
+#  Unless required by applicable law or agreed to in writing,
+#  software distributed under the License is distributed on an
+#  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+#  KIND, either express or implied.  See the License for the
+#  specific language governing permissions and limitations
+#  under the License.
+
 from __future__ import unicode_literals
 import logging
 
-from ..transport import Transport
-from ..exceptions import TransportError
-from ..compat import string_types, urlparse, unquote
+from ..transport import Transport, TransportError
+from .utils import query_params, _make_path, SKIP_IN_PATH, _bulk_body, _normalize_hosts
+
+from .async_search import AsyncSearchClient
+from .autoscaling import AutoscalingClient
+from .cat import CatClient
+from .cluster import ClusterClient
+from .dangling_indices import DanglingIndicesClient
 from .indices import IndicesClient
 from .ingest import IngestClient
-from .cluster import ClusterClient
-from .cat import CatClient
 from .nodes import NodesClient
 from .remote import RemoteClient
 from .snapshot import SnapshotClient
 from .tasks import TasksClient
-from .xpack import XPackClient
-from .utils import query_params, _make_path, SKIP_IN_PATH, _bulk_body
 
 # xpack APIs
-from .async_search import AsyncSearchClient
-from .autoscaling import AutoscalingClient
+from .xpack import XPackClient
 from .ccr import CcrClient
 from .data_frame import Data_FrameClient
 from .deprecation import DeprecationClient
+from .enrich import EnrichClient
 from .eql import EqlClient
 from .graph import GraphClient
 from .ilm import IlmClient
@@ -30,63 +48,18 @@ from .migration import MigrationClient
 from .ml import MlClient
 from .monitoring import MonitoringClient
 from .rollup import RollupClient
+from .searchable_snapshots import SearchableSnapshotsClient
 from .security import SecurityClient
+from .slm import SlmClient
 from .sql import SqlClient
 from .ssl import SslClient
-from .watcher import WatcherClient
-from .enrich import EnrichClient
-from .slm import SlmClient
 from .transform import TransformClient
+from .watcher import WatcherClient
 
 
 logger = logging.getLogger("elasticsearch")
 
 
-def _normalize_hosts(hosts):
-    """
-    Helper function to transform hosts argument to
-    :class:`~elasticsearch.Elasticsearch` to a list of dicts.
-    """
-    # if hosts are empty, just defer to defaults down the line
-    if hosts is None:
-        return [{}]
-
-    # passed in just one string
-    if isinstance(hosts, string_types):
-        hosts = [hosts]
-
-    out = []
-    # normalize hosts to dicts
-    for host in hosts:
-        if isinstance(host, string_types):
-            if "://" not in host:
-                host = "//%s" % host
-
-            parsed_url = urlparse(host)
-            h = {"host": parsed_url.hostname}
-
-            if parsed_url.port:
-                h["port"] = parsed_url.port
-
-            if parsed_url.scheme == "https":
-                h["port"] = parsed_url.port or 443
-                h["use_ssl"] = True
-
-            if parsed_url.username or parsed_url.password:
-                h["http_auth"] = "%s:%s" % (
-                    unquote(parsed_url.username),
-                    unquote(parsed_url.password),
-                )
-
-            if parsed_url.path and parsed_url.path != "/":
-                h["url_prefix"] = parsed_url.path
-
-            out.append(h)
-        else:
-            out.append(host)
-    return out
-
-
 class Elasticsearch(object):
     """
     Elasticsearch low-level client. Provides a straightforward mapping from
@@ -230,21 +203,23 @@ class Elasticsearch(object):
         self.transport = transport_class(_normalize_hosts(hosts), **kwargs)
 
         # namespaced clients for compatibility with API names
+        self.async_search = AsyncSearchClient(self)
+        self.autoscaling = AutoscalingClient(self)
+        self.cat = CatClient(self)
+        self.cluster = ClusterClient(self)
+        self.dangling_indices = DanglingIndicesClient(self)
         self.indices = IndicesClient(self)
         self.ingest = IngestClient(self)
-        self.cluster = ClusterClient(self)
-        self.cat = CatClient(self)
         self.nodes = NodesClient(self)
         self.remote = RemoteClient(self)
         self.snapshot = SnapshotClient(self)
         self.tasks = TasksClient(self)
 
         self.xpack = XPackClient(self)
-        self.async_search = AsyncSearchClient(self)
-        self.autoscaling = AutoscalingClient(self)
         self.ccr = CcrClient(self)
         self.data_frame = Data_FrameClient(self)
         self.deprecation = DeprecationClient(self)
+        self.enrich = EnrichClient(self)
         self.eql = EqlClient(self)
         self.graph = GraphClient(self)
         self.ilm = IlmClient(self)
@@ -254,13 +229,13 @@ class Elasticsearch(object):
         self.ml = MlClient(self)
         self.monitoring = MonitoringClient(self)
         self.rollup = RollupClient(self)
+        self.searchable_snapshots = SearchableSnapshotsClient(self)
         self.security = SecurityClient(self)
+        self.slm = SlmClient(self)
         self.sql = SqlClient(self)
         self.ssl = SslClient(self)
-        self.watcher = WatcherClient(self)
-        self.enrich = EnrichClient(self)
-        self.slm = SlmClient(self)
         self.transform = TransformClient(self)
+        self.watcher = WatcherClient(self)
 
     def __repr__(self):
         try:
@@ -274,12 +249,25 @@ class Elasticsearch(object):
             # probably operating on custom transport and connection_pool, ignore
             return super(Elasticsearch, self).__repr__()
 
+    def __enter__(self):
+        if hasattr(self.transport, "_async_call"):
+            self.transport._async_call()
+        return self
+
+    def __exit__(self, *_):
+        self.close()
+
+    def close(self):
+        """Closes the Transport and all internal connections"""
+        self.transport.close()
+
     # AUTO-GENERATED-API-DEFINITIONS #
     @query_params()
     def ping(self, params=None, headers=None):
         """
         Returns whether the cluster is running.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/index.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/index.html>`_
         """
         try:
             return self.transport.perform_request(
@@ -292,7 +280,8 @@ class Elasticsearch(object):
     def info(self, params=None, headers=None):
         """
         Returns basic information about the cluster.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/index.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/index.html>`_
         """
         return self.transport.perform_request(
             "GET", "/", params=params, headers=headers
@@ -311,7 +300,8 @@ class Elasticsearch(object):
         """
         Creates a new document in the index.  Returns a 409 response when a document
         with a same ID already exists in the index.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/master/docs-index_.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/docs-index_.html>`_
 
         :arg index: The name of the index
         :arg id: Document ID
@@ -339,14 +329,12 @@ class Elasticsearch(object):
                 raise ValueError("Empty value passed for a required argument.")
 
         if doc_type in SKIP_IN_PATH:
-            doc_type = "_doc"
+            path = _make_path(index, "_create", id)
+        else:
+            path = _make_path(index, doc_type, id, "_create")
 
         return self.transport.perform_request(
-            "PUT",
-            _make_path(index, doc_type, id, "_create"),
-            params=params,
-            headers=headers,
-            body=body,
+            "PUT", path, params=params, headers=headers, body=body
         )
 
     @query_params(
@@ -355,6 +343,7 @@ class Elasticsearch(object):
         "op_type",
         "pipeline",
         "refresh",
+        "require_alias",
         "routing",
         "timeout",
         "version",
@@ -364,7 +353,8 @@ class Elasticsearch(object):
     def index(self, index, body, doc_type=None, id=None, params=None, headers=None):
         """
         Creates or updates a document in an index.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/master/docs-index_.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/docs-index_.html>`_
 
         :arg index: The name of the index
         :arg body: The document
@@ -385,6 +375,8 @@ class Elasticsearch(object):
             this operation visible to search, if `wait_for` then wait for a refresh
             to make this operation visible to search, if `false` (the default) then
             do nothing with refreshes.  Valid choices: true, false, wait_for
+        :arg require_alias: When true, requires destination to be an
+            alias. Default is false
         :arg routing: Specific routing value
         :arg timeout: Explicit operation timeout
         :arg version: Explicit version number for concurrency control
@@ -417,6 +409,7 @@ class Elasticsearch(object):
         "_source_includes",
         "pipeline",
         "refresh",
+        "require_alias",
         "routing",
         "timeout",
         "wait_for_active_shards",
@@ -424,7 +417,8 @@ class Elasticsearch(object):
     def bulk(self, body, index=None, doc_type=None, params=None, headers=None):
         """
         Allows to perform multiple index/update/delete operations in a single request.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/master/docs-bulk.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/docs-bulk.html>`_
 
         :arg body: The operation definition and data (action-data
             pairs), separated by newlines
@@ -444,6 +438,8 @@ class Elasticsearch(object):
             this operation visible to search, if `wait_for` then wait for a refresh
             to make this operation visible to search, if `false` (the default) then
             do nothing with refreshes.  Valid choices: true, false, wait_for
+        :arg require_alias: Sets require_alias for all incoming
+            documents. Defaults to unset (false)
         :arg routing: Specific routing value
         :arg timeout: Explicit operation timeout
         :arg wait_for_active_shards: Sets the number of shard copies
@@ -468,7 +464,8 @@ class Elasticsearch(object):
     def clear_scroll(self, body=None, scroll_id=None, params=None, headers=None):
         """
         Explicitly clears the search context for a scroll.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/master/search-request-body.html#_clear_scroll_api>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/clear-scroll-api.html>`_
 
         :arg body: A comma-separated list of scroll IDs to clear if none
             was specified via the scroll_id parameter
@@ -504,7 +501,8 @@ class Elasticsearch(object):
     def count(self, body=None, index=None, doc_type=None, params=None, headers=None):
         """
         Returns number of documents matching a query.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/master/search-count.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/search-count.html>`_
 
         :arg body: A query to restrict the results specified with the
             Query DSL (optional)
@@ -561,7 +559,8 @@ class Elasticsearch(object):
     def delete(self, index, id, doc_type=None, params=None, headers=None):
         """
         Removes a document from the index.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/master/docs-delete.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/docs-delete.html>`_
 
         :arg index: The name of the index
         :arg id: The document ID
@@ -636,7 +635,8 @@ class Elasticsearch(object):
     def delete_by_query(self, index, body, doc_type=None, params=None, headers=None):
         """
         Deletes documents matching the provided query.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/master/docs-delete-by-query.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/docs-delete-by-query.html>`_
 
         :arg index: A comma-separated list of index names to search; use
             `_all` or empty string to perform the operation on all indices
@@ -732,7 +732,8 @@ class Elasticsearch(object):
         """
         Changes the number of requests per second for a particular Delete By Query
         operation.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/docs-delete-by-query.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/docs-delete-by-query.html>`_
 
         :arg task_id: The task id to rethrottle
         :arg requests_per_second: The throttle to set on this request in
@@ -752,7 +753,8 @@ class Elasticsearch(object):
     def delete_script(self, id, params=None, headers=None):
         """
         Deletes a script.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/master/modules-scripting.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/modules-scripting.html>`_
 
         :arg id: Script ID
         :arg master_timeout: Specify timeout for connection to master
@@ -780,7 +782,8 @@ class Elasticsearch(object):
     def exists(self, index, id, doc_type=None, params=None, headers=None):
         """
         Returns information about whether a document exists in an index.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/master/docs-get.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/docs-get.html>`_
 
         :arg index: The name of the index
         :arg id: The document ID
@@ -830,7 +833,8 @@ class Elasticsearch(object):
     def exists_source(self, index, id, doc_type=None, params=None, headers=None):
         """
         Returns information about whether a document source exists in an index.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/master/docs-get.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/docs-get.html>`_
 
         :arg index: The name of the index
         :arg id: The document ID
@@ -857,11 +861,13 @@ class Elasticsearch(object):
             if param in SKIP_IN_PATH:
                 raise ValueError("Empty value passed for a required argument.")
 
+        if doc_type in SKIP_IN_PATH:
+            path = _make_path(index, "_source", id)
+        else:
+            path = _make_path(index, doc_type, id, "_source")
+
         return self.transport.perform_request(
-            "HEAD",
-            _make_path(index, doc_type, id, "_source"),
-            params=params,
-            headers=headers,
+            "HEAD", path, params=params, headers=headers
         )
 
     @query_params(
@@ -881,7 +887,8 @@ class Elasticsearch(object):
     def explain(self, index, id, body=None, doc_type=None, params=None, headers=None):
         """
         Returns information about why a specific matches (or doesn't match) a query.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/master/search-explain.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/search-explain.html>`_
 
         :arg index: The name of the index
         :arg id: The document ID
@@ -914,14 +921,12 @@ class Elasticsearch(object):
                 raise ValueError("Empty value passed for a required argument.")
 
         if doc_type in SKIP_IN_PATH:
-            doc_type = "_doc"
+            path = _make_path(index, "_explain", id)
+        else:
+            path = _make_path(index, doc_type, id, "_explain")
 
         return self.transport.perform_request(
-            "POST",
-            _make_path(index, doc_type, id, "_explain"),
-            params=params,
-            headers=headers,
-            body=body,
+            "POST", path, params=params, headers=headers, body=body
         )
 
     @query_params(
@@ -931,12 +936,14 @@ class Elasticsearch(object):
         "ignore_unavailable",
         "include_unmapped",
     )
-    def field_caps(self, index=None, params=None, headers=None):
+    def field_caps(self, body=None, index=None, params=None, headers=None):
         """
         Returns the information about the capabilities of fields among multiple
         indices.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/master/search-field-caps.html>`_
 
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/search-field-caps.html>`_
+
+        :arg body: An index filter specified with the Query DSL
         :arg index: A comma-separated list of index names; use `_all` or
             empty string to perform the operation on all indices
         :arg allow_no_indices: Whether to ignore if a wildcard indices
@@ -952,7 +959,11 @@ class Elasticsearch(object):
             be included in the response.
         """
         return self.transport.perform_request(
-            "GET", _make_path(index, "_field_caps"), params=params, headers=headers
+            "POST",
+            _make_path(index, "_field_caps"),
+            params=params,
+            headers=headers,
+            body=body,
         )
 
     @query_params(
@@ -970,7 +981,8 @@ class Elasticsearch(object):
     def get(self, index, id, doc_type=None, params=None, headers=None):
         """
         Returns a document.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/master/docs-get.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/docs-get.html>`_
 
         :arg index: The name of the index
         :arg id: The document ID
@@ -1010,7 +1022,8 @@ class Elasticsearch(object):
     def get_script(self, id, params=None, headers=None):
         """
         Returns a script.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/master/modules-scripting.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/modules-scripting.html>`_
 
         :arg id: Script ID
         :arg master_timeout: Specify timeout for connection to master
@@ -1036,7 +1049,8 @@ class Elasticsearch(object):
     def get_source(self, index, id, doc_type=None, params=None, headers=None):
         """
         Returns the source of a document.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/master/docs-get.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/docs-get.html>`_
 
         :arg index: The name of the index
         :arg id: The document ID
@@ -1064,13 +1078,12 @@ class Elasticsearch(object):
                 raise ValueError("Empty value passed for a required argument.")
 
         if doc_type in SKIP_IN_PATH:
-            doc_type = "_doc"
+            path = _make_path(index, "_source", id)
+        else:
+            path = _make_path(index, doc_type, id, "_source")
 
         return self.transport.perform_request(
-            "GET",
-            _make_path(index, doc_type, id, "_source"),
-            params=params,
-            headers=headers,
+            "GET", path, params=params, headers=headers
         )
 
     @query_params(
@@ -1086,7 +1099,8 @@ class Elasticsearch(object):
     def mget(self, body, index=None, doc_type=None, params=None, headers=None):
         """
         Allows to get multiple documents in one request.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/master/docs-multi-get.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/docs-multi-get.html>`_
 
         :arg body: Document identifiers; can be either `docs`
             (containing full document information) or `ids` (when index and type is
@@ -1132,7 +1146,8 @@ class Elasticsearch(object):
     def msearch(self, body, index=None, doc_type=None, params=None, headers=None):
         """
         Allows to execute several search operations in one request.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/master/search-multi-search.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/search-multi-search.html>`_
 
         :arg body: The request definitions (metadata-search request
             definition pairs), separated by newlines
@@ -1188,7 +1203,8 @@ class Elasticsearch(object):
     ):
         """
         Allows to execute several search template operations in one request.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/search-multi-search.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/search-multi-search.html>`_
 
         :arg body: The request definitions (metadata-search request
             definition pairs), separated by newlines
@@ -1240,7 +1256,8 @@ class Elasticsearch(object):
     ):
         """
         Returns multiple termvectors in one request.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/master/docs-multi-termvectors.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/docs-multi-termvectors.html>`_
 
         :arg body: Define ids, documents, parameters or a list of
             parameters per document here. You must at least provide a list of
@@ -1279,19 +1296,21 @@ class Elasticsearch(object):
         :arg version_type: Specific version type  Valid choices:
             internal, external, external_gte, force
         """
+        if doc_type in SKIP_IN_PATH:
+            path = _make_path(index, "_mtermvectors")
+        else:
+            path = _make_path(index, doc_type, "_mtermvectors")
+
         return self.transport.perform_request(
-            "POST",
-            _make_path(index, doc_type, "_mtermvectors"),
-            params=params,
-            headers=headers,
-            body=body,
+            "POST", path, params=params, headers=headers, body=body
         )
 
     @query_params("master_timeout", "timeout")
     def put_script(self, id, body, context=None, params=None, headers=None):
         """
         Creates or updates a script.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/master/modules-scripting.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/modules-scripting.html>`_
 
         :arg id: Script ID
         :arg body: The document
@@ -1318,7 +1337,13 @@ class Elasticsearch(object):
         """
         Allows to evaluate the quality of ranked search results over a set of typical
         search queries
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/master/search-rank-eval.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/search-rank-eval.html>`_
+
+        .. warning::
+
+            This API is **experimental** so may include breaking changes
+            or be removed in a future version
 
         :arg body: The ranking evaluation search definition, including
             search requests, document ratings and ranking metric definition.
@@ -1361,7 +1386,8 @@ class Elasticsearch(object):
         Allows to copy documents from one index to another, optionally filtering the
         source documents by a query, changing the destination index settings, or
         fetching the documents from a remote cluster.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/master/docs-reindex.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/docs-reindex.html>`_
 
         :arg body: The search definition using the Query DSL and the
             prototype for the index request.
@@ -1396,7 +1422,8 @@ class Elasticsearch(object):
     def reindex_rethrottle(self, task_id, params=None, headers=None):
         """
         Changes the number of requests per second for a particular Reindex operation.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/master/docs-reindex.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/docs-reindex.html>`_
 
         :arg task_id: The task id to rethrottle
         :arg requests_per_second: The throttle to set on this request in
@@ -1416,7 +1443,8 @@ class Elasticsearch(object):
     def render_search_template(self, body=None, id=None, params=None, headers=None):
         """
         Allows to use the Mustache language to pre-render a search definition.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/search-template.html#_validating_templates>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/search-template.html#_validating_templates>`_
 
         :arg body: The search definition template and its params
         :arg id: The id of the stored search template
@@ -1433,8 +1461,14 @@ class Elasticsearch(object):
     def scripts_painless_execute(self, body=None, params=None, headers=None):
         """
         Allows an arbitrary script to be executed and a result to be returned
+
         `<https://www.elastic.co/guide/en/elasticsearch/painless/master/painless-execute-api.html>`_
 
+        .. warning::
+
+            This API is **experimental** so may include breaking changes
+            or be removed in a future version
+
         :arg body: The script to execute
         """
         return self.transport.perform_request(
@@ -1449,7 +1483,8 @@ class Elasticsearch(object):
     def scroll(self, body=None, scroll_id=None, params=None, headers=None):
         """
         Allows to retrieve a large numbers of results from a single search request.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/master/search-request-body.html#request-body-search-scroll>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/search-request-body.html#request-body-search-scroll>`_
 
         :arg body: The scroll ID if not passed by URL or query
             parameter.
@@ -1517,7 +1552,8 @@ class Elasticsearch(object):
     def search(self, body=None, index=None, doc_type=None, params=None, headers=None):
         """
         Returns results matching a query.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/master/search-search.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/search-search.html>`_
 
         :arg body: The search definition using the Query DSL
         :arg index: A comma-separated list of index names to search; use
@@ -1639,7 +1675,8 @@ class Elasticsearch(object):
         """
         Returns information about the indices and shards that a search request would be
         executed against.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/master/search-shards.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/search-shards.html>`_
 
         :arg index: A comma-separated list of index names to search; use
             `_all` or empty string to perform the operation on all indices
@@ -1681,7 +1718,8 @@ class Elasticsearch(object):
     ):
         """
         Allows to use the Mustache language to pre-render a search definition.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/search-template.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/search-template.html>`_
 
         :arg body: The search definition template and its params
         :arg index: A comma-separated list of index names to search; use
@@ -1747,7 +1785,8 @@ class Elasticsearch(object):
         """
         Returns information and statistics about terms in the fields of a particular
         document.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/master/docs-termvectors.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/docs-termvectors.html>`_
 
         :arg index: The index in which the document resides.
         :arg body: Define parameters and or supply a document to get
@@ -1780,14 +1819,12 @@ class Elasticsearch(object):
             raise ValueError("Empty value passed for a required argument 'index'.")
 
         if doc_type in SKIP_IN_PATH:
-            doc_type = "_doc"
+            path = _make_path(index, "_termvectors", id)
+        else:
+            path = _make_path(index, doc_type, id, "_termvectors")
 
         return self.transport.perform_request(
-            "POST",
-            _make_path(index, doc_type, id, "_termvectors"),
-            params=params,
-            headers=headers,
-            body=body,
+            "POST", path, params=params, headers=headers, body=body
         )
 
     @query_params(
@@ -1798,6 +1835,7 @@ class Elasticsearch(object):
         "if_seq_no",
         "lang",
         "refresh",
+        "require_alias",
         "retry_on_conflict",
         "routing",
         "timeout",
@@ -1806,7 +1844,8 @@ class Elasticsearch(object):
     def update(self, index, id, body, doc_type=None, params=None, headers=None):
         """
         Updates a document with a script or partial document.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/master/docs-update.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/docs-update.html>`_
 
         :arg index: The name of the index
         :arg id: Document ID
@@ -1830,6 +1869,8 @@ class Elasticsearch(object):
             this operation visible to search, if `wait_for` then wait for a refresh
             to make this operation visible to search, if `false` (the default) then
             do nothing with refreshes.  Valid choices: true, false, wait_for
+        :arg require_alias: When true, requires destination is an alias.
+            Default is false
         :arg retry_on_conflict: Specify how many times should the
             operation be retried when a conflict occurs (default: 0)
         :arg routing: Specific routing value
@@ -1845,14 +1886,12 @@ class Elasticsearch(object):
                 raise ValueError("Empty value passed for a required argument.")
 
         if doc_type in SKIP_IN_PATH:
-            doc_type = "_doc"
+            path = _make_path(index, "_update", id)
+        else:
+            path = _make_path(index, doc_type, id, "_update")
 
         return self.transport.perform_request(
-            "POST",
-            _make_path(index, doc_type, id, "_update"),
-            params=params,
-            headers=headers,
-            body=body,
+            "POST", path, params=params, headers=headers, body=body
         )
 
     @query_params(
@@ -1898,7 +1937,8 @@ class Elasticsearch(object):
         """
         Performs an update on every document in the index without changing the source,
         for example to pick up a mapping change.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/master/docs-update-by-query.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/docs-update-by-query.html>`_
 
         :arg index: A comma-separated list of index names to search; use
             `_all` or empty string to perform the operation on all indices
@@ -1997,7 +2037,8 @@ class Elasticsearch(object):
         """
         Changes the number of requests per second for a particular Update By Query
         operation.
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/docs-update-by-query.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/docs-update-by-query.html>`_
 
         :arg task_id: The task id to rethrottle
         :arg requests_per_second: The throttle to set on this request in
@@ -2017,7 +2058,13 @@ class Elasticsearch(object):
     def get_script_context(self, params=None, headers=None):
         """
         Returns all script contexts.
+
         `<https://www.elastic.co/guide/en/elasticsearch/painless/master/painless-contexts.html>`_
+
+        .. warning::
+
+            This API is **experimental** so may include breaking changes
+            or be removed in a future version
         """
         return self.transport.perform_request(
             "GET", "/_script_context", params=params, headers=headers
@@ -2027,8 +2074,53 @@ class Elasticsearch(object):
     def get_script_languages(self, params=None, headers=None):
         """
         Returns available script types, languages and contexts
-        `<https://www.elastic.co/guide/en/elasticsearch/reference/master/modules-scripting.html>`_
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/modules-scripting.html>`_
+
+        .. warning::
+
+            This API is **experimental** so may include breaking changes
+            or be removed in a future version
         """
         return self.transport.perform_request(
             "GET", "/_script_language", params=params, headers=headers
         )
+
+    @query_params()
+    def close_point_in_time(self, body=None, params=None, headers=None):
+        """
+        Close a point in time
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/point-in-time-api.html>`_
+
+        :arg body: a point-in-time id to close
+        """
+        return self.transport.perform_request(
+            "DELETE", "/_pit", params=params, headers=headers, body=body
+        )
+
+    @query_params(
+        "expand_wildcards", "ignore_unavailable", "keep_alive", "preference", "routing"
+    )
+    def open_point_in_time(self, index=None, params=None, headers=None):
+        """
+        Open a point in time that can be used in subsequent searches
+
+        `<https://www.elastic.co/guide/en/elasticsearch/reference/7.10/point-in-time-api.html>`_
+
+        :arg index: A comma-separated list of index names to open point
+            in time; use `_all` or empty string to perform the operation on all
+            indices
+        :arg expand_wildcards: Whether to expand wildcard expression to
+            concrete indices that are open, closed or both.  Valid choices: open,
+            closed, hidden, none, all  Default: open
+        :arg ignore_unavailable: Whether specified concrete indices
+            should be ignored when unavailable (missing or closed)
+        :arg keep_alive: Specific the time to live for the point in time
+        :arg preference: Specify the node or shard the operation should
+            be performed on (default: random)
+        :arg routing: Specific routing value
+        """
+        return self.transport.perform_request(
+            "POST", _make_path(index, "_pit"), params=params, headers=headers
+        )
